import make_ancestors
import os
import pytest


def test___build_directed_grapth():

    # Change the value of DEPEND file to run tests. We cannot use DEPEND as file name because
    # it is reserved in SAS
    make_ancestors.DEPEND_FILE_NAME = "DEPEND_TEST_FILE"

    # Prepare test fixture
    packages_dir = os.path.join(os.getcwd(), "test", "test_make_ancestors.data", "packages")

    # Expected set of packages
    expected_dependency_set = {
        ("package_a", "package_d"), 
        ("package_b", "package_a"), 
        ("package_c", "package_a"), 
        ("package_c", "package_b")
    }

    # Actual set of dependencies
    actual_dependency_set = make_ancestors.__build_dependencies_list(packages_dir)
    
    # Compare sets
    assert expected_dependency_set == actual_dependency_set, "dependencies are not the same"

def test_get_ancestors_build_list_tsorted():
    # Test only one element in the grapth
    dependency_set = {}
    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "a")
    assert ["a"] == ancestors_tsorted, "dependencies are not the same"

    # Test two elements in the grapth
    dependency_set = {("b", "a")}
    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "a")
    assert ["a", "b"] == ancestors_tsorted, "dependencies are not the same"

    dependency_set = {("b", "a")}
    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "b")
    assert ["b"] == ancestors_tsorted, "dependencies are not the same"

    # Test two subgrapths disconected
    dependency_set = {("b", "a"), ("d", "c")}
    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "c")
    assert ["c", "d"] == ancestors_tsorted, "dependencies are not the same"

    # Test example from https://en.wikipedia.org/wiki/Topological_sorting
    dependency_set = {
        ("5", "11"), 
        ("7", "11"), ("7", "8"),
        ("3", "8"),  ("3", "10"),
        ("11", "2"), ("11", "9"), ("11", "10"),
        ("8", "9")
    }
    valid_topological_sorts__source_2 = [
        ['2', '11', '7', '5'],
        ['2', '11', '5', '7']
    ]
    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "2")
    assert ancestors_tsorted in valid_topological_sorts__source_2, "dependencies are not the same"

    some_valid_topological_sorts__source_9 = [
        ['9', '8', '3', '11', '5', '7'],
        ['9', '8', '3', '11', '7', '5'],
        ['9', '8', '11', '3', '7', '5'],
        ['9', '11', '8', '5', '3', '7'],
        ['9', '11', '8', '7', '5', '3'],
        ['9', '11', '8', '3', '7', '5'],
        ['9', '11', '8', '5', '7', '3'],
        ['9', '11', '5', '8', '7', '3']
    ]

    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "9")
    assert ancestors_tsorted in some_valid_topological_sorts__source_9, "dependencies are not the same"

    valid_topological_sorts__source_8 = [
        ['8', '3', '7'],
        ['8', '7', '3']
    ]
    ancestors_tsorted = make_ancestors.get_ancestors_build_list_tsorted(dependency_set, "8")
    assert ancestors_tsorted in valid_topological_sorts__source_8, "dependencies are not the same"


def test_execute_make_for_each_package():

    # Prepare test fixture
    packages_dir = os.path.join(os.getcwd(), "test", "test_make_ancestors.data", "packages")

    # Execute 'make ckeck' for all package_list
    package_list = ["package_d", "package_a", "package_b", "package_c"]
    make_ancestors.execute_make_for_each_package(packages_dir, package_list)

    # Execute 'make ckeck' with wrong packages_dir
    with pytest.raises(Exception):
        make_ancestors.execute_make_for_each_package("/tmp", ["package_a"])

    # Execute 'make ckeck' for an unexisting package
    with pytest.raises(Exception):
        make_ancestors.execute_make_for_each_package(packages_dir, ["no_package"])
